-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- 2022 HW design:  Mark D. Aagaard, Nusa Zidaric
-- 2022 VHDL implementation: Nusa Zidaric
-- note overlap with ACE modules at
-- ( http://uwaterloo.ca/communications-security-lab/lwc/ace )

-- SpoC submitters/designers:
-- Riham AlTawy
-- Guang Gong
-- Morgan He
-- Ashwin Jha
-- Kalikinkar Mandal
-- Mridul Nandi
-- Raghvendra Rohit

-- http://uwaterloo.ca/communications-security-lab/lwc/spoc

-- This is a human-readable summary of (and not a substitute for) the license.
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.
---------------------------
------------------------------------------
-- control signals for DEC padding
------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.spoc_128_pkg.all;
use work.ctrl_pkg.all;

entity ctrl_pad_to_en is
  port
    ( i_padding 	: in  pad_ty
    ; en_ctrl   	: out en_ty
    ; pad_ctrl   	: out padxor_ty
   );
end entity;

architecture rtl of ctrl_pad_to_en is

signal en : en_ty;
signal pad : padxor_ty;

begin

   en_ctrl 	<= en;
   pad_ctrl <= pad;

   en      <= "1111111111111111" when i_padding = "0000" else
              "1000000000000000" when i_padding = "0001" else
              "1100000000000000" when i_padding = "0010" else
              "1110000000000000" when i_padding = "0011" else
              "1111000000000000" when i_padding = "0100" else
              "1111100000000000" when i_padding = "0101" else
              "1111110000000000" when i_padding = "0110" else
              "1111111000000000" when i_padding = "0111" else
              "1111111100000000" when i_padding = "1000" else
              "1111111110000000" when i_padding = "1001" else
              "1111111111000000" when i_padding = "1010" else
              "1111111111100000" when i_padding = "1011" else
              "1111111111110000" when i_padding = "1100" else
              "1111111111111000" when i_padding = "1101" else
              "1111111111111100" when i_padding = "1110" else
              "1111111111111110" ;

   padxors: for j in 0 to 14  generate
  		 pad(j) <= en(j) xor  en(j+1);
 	end generate padxors;

end architecture;
