-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- 2022 HW design:  Mark D. Aagaard, Nusa Zidaric
-- 2022 VHDL implementation: Nusa Zidaric
-- note overlap with ACE modules at
-- ( http://uwaterloo.ca/communications-security-lab/lwc/ace )

-- SpoC submitters/designers:
-- Riham AlTawy
-- Guang Gong
-- Morgan He
-- Ashwin Jha
-- Kalikinkar Mandal
-- Mridul Nandi
-- Raghvendra Rohit

-- http://uwaterloo.ca/communications-security-lab/lwc/spoc

-- This is a human-readable summary of (and not a substitute for) the license.
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.
---------------------------

library ieee;
use ieee.std_logic_1164.all;
use work.spoc_128_pkg.all;


entity lfsr_c is
  port
    ( clk          : in std_logic
    ; lfsr_c_en    : in std_logic
    ; lfsr_c_reset : in std_logic
    ; o_const      : out lfsr_c_output
   );
end lfsr_c;

architecture rtl of lfsr_c is
  signal sa: std_logic_vector(lfsr_c_sz - 1 downto 0);
  signal xa: std_logic_vector(lfsr_c_sz + 1 downto 0);
  signal ya: std_logic_vector(lfsr_c_sz - 1 downto 0);
  signal wa: std_logic_vector(lfsr_c_sz - 1 downto 0);
  signal va: std_logic_vector(lfsr_c_sz - 1 downto 0);
begin

-- 18 output bits for the constants
  o_const(0) <= sa(0);   -- rc0
  o_const(1) <= sa(1);   -- rc1
  -- sc0
  o_const(2) <= ya(2); -- s28
  o_const(3) <= xa(6); -- s26
  o_const(4) <= xa(4); -- s24
  o_const(5) <= xa(2); -- s22
  o_const(6) <= sa(6); -- s20
  o_const(7) <= sa(4); -- s18
  o_const(8) <= sa(2); -- s16
  o_const(9) <= sa(0); -- s14
  -- sc1
  o_const(10) <= ya(3); -- s29
  o_const(11) <= ya(1); -- s27
  o_const(12) <= xa(5); -- s25
  o_const(13) <= xa(3); -- s23
  o_const(14) <= xa(1); -- s21
  o_const(15) <= sa(5); -- s19
  o_const(16) <= sa(3); -- s17
  o_const(17) <= sa(1); -- s15

-- just rename signal
  xa(0) <= sa(lfsr_c_sz-1);
  ya(0) <= xa(lfsr_c_sz-1);

-- for updates and outputs
  xa(6 downto 1) <= sa(6 downto 1) xor sa(5 downto 0);
  ya(6 downto 1) <= xa(6 downto 1) xor xa(5 downto 0);

  lfsr_shift: for i in lfsr_c_sz-1 downto 0 generate
    lfsr_step: process(clk) begin
      if rising_edge(clk) then
        if lfsr_c_reset ='1' then
          sa(i) <= '1';
        elsif lfsr_c_en ='1' then
          if i<=4 then
            sa(i) <= sa(i+2);
          else
            sa(i) <= xa(i-4);
          end if;
        end if;
      end if;
    end process;
  end generate lfsr_shift;

end;
