-- This work is licensed under a Creative Commons
-- Attribution-NonCommercial-ShareAlike 4.0 International License.
-- http://creativecommons.org/licenses/by-nc-sa/4.0

-- SpoC-128_sLiSCP-light-[256]

-- 2022 HW design:  Mark D. Aagaard, Nusa Zidaric
-- 2022 VHDL implementation: Nusa Zidaric
-- note overlap with ACE modules at
-- ( http://uwaterloo.ca/communications-security-lab/lwc/ace )

-- SpoC submitters/designers:
-- Riham AlTawy
-- Guang Gong
-- Morgan He
-- Ashwin Jha
-- Kalikinkar Mandal
-- Mridul Nandi
-- Raghvendra Rohit

-- http://uwaterloo.ca/communications-security-lab/lwc/spoc

-- This is a human-readable summary of (and not a substitute for) the license.
-- You are free to:

--     Share:  copy and redistribute the material in any medium or format
--     Adapt:  remix, transform, and build upon the material

--     The licensor cannot revoke these freedoms as long as you follow
--     the license terms.

-- Under the following terms:

--     Attribution - You must give appropriate credit, provide a link to
--     the license, and indicate if changes were made. You may do so in
--     any reasonable manner, but not in any way that suggests the
--     licensor endorses you or your use.

--     NonCommercial - You may not use the material for commercial
--     purposes.

--     ShareAlike - If you remix, transform, or build upon the material,
--     you must distribute your contributions under the same license as
--     the original.

--     No additional restrictions - You may not apply legal terms or
--     technological measures that legally restrict others from doing
--     anything the license permits.
---------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.spoc_128_pkg.all;
use work.ctrl_pkg.all;

entity spoc_128 is
  port
    ( clk       	 : in  std_logic
     ; reset     	 : in  std_logic
     ; pre_mode    : in  mode_ty
     ; pre_ds_ctrl : in  ds_ctrl_ty
     ; pre_padding : in  pad_ty
     ; pre_valid   : in  std_logic
     ; post_ready  : in  std_logic
     ; pre_data    : in  double_word
     ; post_data   : out double_word
     ; post_valid  : out std_logic
     ; pre_ready   : out std_logic
      );
end entity;

architecture rtl of spoc_128 is
  signal control    : dp_op_ty;
  signal k_count    : std_logic;
  signal new_ctrl   : en_ty;
  signal pad_ctrl   : padxor_ty;
begin

  u_dp :  entity work.dp port map
      ( clk, k_count, control, new_ctrl, pad_ctrl
      , pre_ds_ctrl,  pre_data, post_data    );

  u_ctrl: entity work.ctrl port map
      ( clk, reset, pre_mode, pre_ds_ctrl, pre_valid, post_ready, pre_padding
      , control, k_count, new_ctrl, pad_ctrl, post_valid, pre_ready     );

end architecture;
